"use strict";
/**
 * link-local-dependencies
 * Install local lerna dependencies
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.linkAllLocalDependencies = exports.linkLocalDependencies = void 0;
const debug_1 = __importDefault(require("debug"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const A = __importStar(require("fp-ts/ReadonlyArray"));
const E = __importStar(require("fp-ts/Either"));
const O = __importStar(require("fp-ts/Option"));
const TE = __importStar(require("fp-ts/TaskEither"));
const PathReporter = __importStar(require("io-ts/lib/PathReporter"));
const function_1 = require("fp-ts/function");
const trace_1 = require("@strong-roots-capital/trace");
const package_manifests_1 = require("@typescript-tools/package-manifests");
const PackageJsonDependencies_1 = require("@typescript-tools/io-ts/dist/lib/PackageJsonDependencies");
const debug = {
    cmd: debug_1.default('link'),
};
const err = function_1.identity;
const findPackageIn = (packages) => (packagePathOrName) => function_1.pipe(O.fromNullable(packages.get(packagePathOrName)), E.fromOption(() => ({
    type: 'unknown package',
    package: packagePathOrName,
})), TE.fromEither, TE.bimap(err, trace_1.trace(debug.cmd, 'Linking dependencies for package')));
const packageManifest = function_1.flow(package_manifests_1.packageManifest, TE.bimap(err, ({ contents }) => contents));
const decodeManifest = (manifest) => function_1.pipe(PackageJsonDependencies_1.PackageJsonDependencies.decode(manifest), E.mapLeft(function_1.flow(errors => PathReporter.failure(errors).join('\n'), error => err({ type: 'unable to decode package manifest', error }))), TE.fromEither);
const internalDependencies = (packages) => (dependencies) => {
    return function_1.pipe(dependencies, A.chain(dependency => function_1.pipe(O.fromNullable(packages.get(dependency)), O.map(A.of), O.getOrElseW(() => A.empty))));
};
const internalPackageDependencies = (packages) => function_1.flow(packageManifest, TE.chain(decodeManifest), TE.map(function_1.flow(PackageJsonDependencies_1.dependencies, A.map(([dependency]) => dependency), internalDependencies(packages))));
const mkdir = (target) => function_1.pipe(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => fs.mkdir(target, { recursive: true }, error => error !== null && error !== undefined
        ? reject(error)
        : resolve(function_1.constVoid())));
}), function_1.flow(E.toError, error => err({ type: 'unable to create directory', path: target, error }))));
/**
 * Note: defaults to no action when `target` already exists.
 * Wonder if I should instead remove the existing file before creating `target`.
 */
const symlink = (target, link) => function_1.pipe(mkdir(path.dirname(link)), TE.chain(() => function_1.pipe(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => fs.symlink(target, link, error => error !== null && error !== undefined
        ? reject(error)
        : resolve(function_1.constVoid())));
}), E.toError), 
// recover from symlink-already-exists error
TE.orElse(error => error.message.startsWith('EEXIST:')
    ? TE.right(function_1.constVoid())
    : TE.left(error)), TE.mapLeft(error => err({
    type: 'unable to create symlink',
    target,
    path: link,
    error,
})))));
const nodeModules = (packageDirectory) => path.resolve(packageDirectory, 'node_modules');
const symlinkPackage = (targetPackage) => (lernaPackage) => {
    const symlinkLocation = path.resolve(nodeModules(targetPackage), lernaPackage.name);
    debug.cmd('Linking ', path.relative(path.dirname(symlinkLocation), lernaPackage.location), 'to', symlinkLocation);
    return symlink(path.relative(path.dirname(symlinkLocation), lernaPackage.location), symlinkLocation);
};
const linkLocalDependencies = (packages) => (packagePathOrName) => function_1.pipe(TE.bindTo('lernaPackage')(findPackageIn(packages)(packagePathOrName)), TE.bind('dependencies', ({ lernaPackage }) => internalPackageDependencies(packages)(lernaPackage)), TE.chain(({ lernaPackage, dependencies }) => TE.traverseArray(symlinkPackage(lernaPackage.location))(dependencies)), TE.map(function_1.constVoid));
exports.linkLocalDependencies = linkLocalDependencies;
const linkAllLocalDependencies = (packagesList, packagesMap) => function_1.pipe(packagesList, TE.traverseArray(function_1.flow(pkg => pkg.location, exports.linkLocalDependencies(packagesMap))), TE.map(function_1.constVoid));
exports.linkAllLocalDependencies = linkAllLocalDependencies;
//# sourceMappingURL=index.js.map